#!/bin/bash

# TODO check log size and rotate if >$SIZE once a month

# IF NEEDED for odd-hour snap tests:
# for snap in `zfs-list-snaps--boojum.sh |grep BJM2HR-W4-D5-H15 |awk '{ print $1 }'`; do zfs destroy -v $snap;done
# for snap in `zfs-list-snaps--boojum.sh |grep BJM2HR-W2-D1 |awk '{ print $1 }'`; do zfs destroy -v $snap;done

# =LLC= © (C)opyright 2017 Boojum Consulting LLC / Dave Bechtel, All rights reserved.
## NOTICE: Only Boojum Consulting LLC personnel may use or redistribute this code,
## Unless given explicit permission by the author - see http://www.boojumconsultingsa.com
#
# this runs in cron.hourly
# and keeps 28-31 days of rotating snapshots taken every (2) hours

# Max snapshots in a month (12*31) = 372
# naming scheme: BJM2HR-Weeknum-DOW-HH
# Ex: BJM2HR-W2-D4-H23 == FEB 08 (WED) @11PM
# Ex: BJM2HR-W3-D6-H20 = July 7 (FRI) @ 8pm

#                            2017
# DOW                   1  2  3  4  5  6  7
#       July                 August              September        
#Su Mo Tu We Th Fr Sa  Su Mo Tu We Th Fr Sa  Su Mo Tu We Th Fr Sa  
#                   1         1  2  3  4  5                  1  2  
# 2  3  4  5  6 *7  8   6  7  8  9 10 11 12   3  4  5  6  7  8  9  
# 9 10 11 12 13 14 15  13 14 15 16 17 18 19  10 11 12 13 14 15 16  
#16 17 18 19 20 21 22  20 21 22 23 24 25 26  17 18 19 20 21 22 23  
#23 24 25 26 27 28 29  27 28 29 30 31        24 25 26 27 28 29 30  
#30 31                

# TODO put in var/log with ROT, this will fill up
logfile=/root/boojum-snapshot-2hours.log

PATH=/sbin:/root/bin:/root/bin/boojum:/usr/local/bin:/usr/local/sbin:/usr/sbin:/bin:/usr/bin:/usr/X11R6/bin: #/usr/NX/bin:/usr/games:

# REF: https://serverfault.com/questions/383666/how-to-determine-number-of-week-of-the-month
#myweeknum=`echo $((($(date +%-d)-1)/7+1))` # July 7 == 1  ## xxx 2017.0903 BUG - thinks this is week 1 when its really week 2!
# FIX: my method: print cal without header, grep line number that contains day
mydayom=`date +%-d` # do not pad field
if [ $mydayom -lt 10 ]; then
#  myweeknum=`cal -h |tail -n +3 |awk 'NF>0' |grep -n "[^0-9]$mydayom " |awk -F: '{print $1}'`
  myweeknum=`cal -h |tail -n +3 |grep -n "[^0-9]$mydayom " |awk -F: '{print $1}'`
else
#  myweeknum=`cal -h |tail -n +3 |grep -n "$mydayom " |awk -F: '{print $1}'`
  myweeknum=`cal -h |tail -n +3 |grep -n $mydayom |awk -F: '{print $1}'`
fi

mydow=`date +%w` # %w day of week (0..6); 0 is Sunday
let mydow=$mydow+1  # cant be 0

myhour=`date +%H` # ex. 09 / 23  

echo "=== START RUN "`date`" - boojum 2-hour 28-31 day rotating snapshot for HOUR $myhour" >> $logfile

RUNPROD=0
# speed things up a bit
# TODO re-enable in prod!
[ $RUNPROD -gt 0 ] && /root/bin/boojum/spinup 2>&1 >> $logfile

echo "= BEGIN Stage 1 - DF before" >> $logfile
df -h -T |head -n 1 >> $logfile # grep z grabs header BUT only if zpool has Z in it!
df -h -T |grep zfs >> $logfile
#zfs list -H -t snapshot |column -t >> $logfile
# xxx removed snaps before 2017.0719 to keep size of logfile down

bjmlabel="BJM2HR-W$myweeknum-D$mydow-H$myhour"
minuterun=`date +%M` # what minute of the hour this script was run

echo "= BEGIN Stage 2 - Destroy existing snaps for all pools - MinOfHour: $minuterun" >> $logfile

for zp in `zpool list |grep -v ALLOC |awk '{ print $1 }'`;do
  echo "= INFO zpool=$zp:WEEK=$myweeknum:DOW=$mydow:HR=$myhour" >> $logfile

# kill ANY preexisting snapshot with this daylabel
#  echo "= Destroy existing snaps for pool $zp - MinOfHour: $minuterun" >> $logfile
  echo "= Destroy existing snaps for pool $zp" >> $logfile

# note -v logs what will be destroyed!
  zfs destroy -R -v $zp@$bjmlabel 2>&1 >>$logfile;rc=$?
  echo "= DESTROYED $zp@$bjmlabel M: $minuterun - RTN CODE:$rc" >> $logfile
[ "$rc" = "1" ] && echo "= NOTE - no pre-existing snapshot detected for $bjmlabel" >> $logfile

# naming scheme: BJM2HR-Weeknum-DOW-HH
# Ex: BJM2HR-W2-D4-H23 == FEB 08 (WED) @11PM
# Ex: BJM2HR-W3-D6-H20 = July 7 (FRI) @ 8pm

# take new snapshot
  zfs snapshot -r $zp@$bjmlabel;rc=$?
  echo "= New snap for $zp@$bjmlabel - RTN CODE:$rc" >> $logfile
#  zfs snapshot -r $zp@boojumDOM$mydnum
done

echo "= BEGIN Stage 3 - Snaps+DF After for week:day==$myweeknum:$mydow" >> $logfile        

#zfs list -H -t snapshot |grep $myday >> $logfile
#zfs list -H -t snapshot |grep $mydnum >> $logfile

#zfs list -H -t snapshot |column -t >> $logfile
# xxx 2017.0824 less info -- ex: BJM2HR-W4-D7
zfs list -H -t snapshot \
  |grep "BJM2HR-W$myweeknum-D$mydow" \
  |column -t \
  >> $logfile

df -h -T |grep zfs >> $logfile

echo "=== END RUN boojum 2-hour/28-31 day snapshot for HOUR $myhour: `date`" >> $logfile 
echo "-------------------------" >> $logfile # separator

exit;

2017.0824 has been running for over a month in VM; more descriptive changes, less log data

2017.0903 FIX for week number of month: REF: https://stackoverflow.com/questions/16875453/check-if-string-contains-non-digit-characters
# Just use a negated character class: grep [^0-9]. This will match any non-numeric character, and not strings composed of only digits
myweeknum=`cal |tail -n +3 |awk 'NF>0' |grep -n '[^0-9]9 ' |awk -F: '{print $1}'`

$ cal
   September 2017     
Su Mo Tu We Th Fr Sa  
                1  2  
 3  4  5  6  7  8  9  
10 11 12 13 14 15 16  
17 18 19 20 21 22 23  
24 25 26 27 28 29 30  
                      
# skip header lines in cal, dont print blank lines, grep LINE NUMBER of non-numeric9+space
$ cal |tail -n +3 |awk 'NF>0' |grep -n '[^0-9]3 '
2: 3  4  5  6  7  8  9  

2017.0910 fix for cal -h
