#!/bin/bash

# FOR UBUNTU 16.04
# set up a "standard" boojum combo server - zfs 

# =LLC= © (C)opyright 2016 Boojum Consulting LLC / Dave Bechtel, All rights reserved.
## NOTICE: Only Boojum Consulting LLC personnel may use or redistribute this code,
## Unless given explicit permission by the author - see http://www.boojumconsultingsa.com
#

logfile=~/doitall-boojum.log

######################################
# define functions

# failexit.mrg
function failexit () {
  echo '! Something failed! Code: '"$1 $2" # code # (and optional description)
  exit $1
}

# Echo something to current console AND log
# Can also handle piped input ( cmd |logecho )
# Warning: Has trouble echoing '*' even when quoted.
function logecho () {
  args=$@

  if [ -z "$args" ]; then
    args='tmp'

    while [ 1 ]; do
      read -e -t2 args

      if [ -n "$args" ]; then
         echo $args |tee -a $logfile;
      else
        break;
      fi
    done

  else
    echo $args |tee -a $logfile;
  fi
} # END FUNC

function setproxy () {
 export http_proxy=http://$1:3128
 export https_proxy=http://$1:3128
 export ftp_proxy=http://$1:3128
 set|grep proxy=
}

# TODO call squid setup script
# setproxy localhost  # if we run squid setup before this

/sbin/ifconfig |grep addr |egrep -v 'inet6|Base|Link' |gawk '{ printf "%s", $2 " | " }' # all on 1 line

# Network test
echo '';echo ''
pingok=1
ping -c 4 google.com || pingok=0

if [ $pingok -eq 1 ]; then
  logecho "OK - PING GOOGLE, proceeding"
else
  failexit 404 "! Ping google FAILED! Cannot proceed - check network"
fi

# bkp
tar czpf /root/etc-orig.tgz /etc

# TODO in VM, make this sdb
usemiltera=`blkid |grep -c spower1tb-e4`
if [ $usemiltera -gt 0 ]; then
  fstabline="LABEL=spower1tb-e4  /mnt/milterausb3 auto  defaults,noatime,rw 0 0"
# only do this if not exist
  grep "spower1tb-e4" /etc/fstab || echo "$fstabline" >> /etc/fstab

  mkdir -pv /mnt/milterausb3
  umount /mnt/milterausb3; touch /mnt/milterausb3/NOTHERE
  mount /mnt/milterausb3

  echo 'Unpacking updated files from milterausb3'
  cd /; tar xzpvf /mnt/milterausb3/boojum/migratcrit.tgz
fi


###########################################
# BEGIN MAIN
df -h |logecho

# make more room on ext4
rootdev=`df / |grep dev |awk '{print $1}'`
tune2fs -m1 $rootdev |logecho
homedev=`df /home |grep dev |awk '{print $1}'`
tune2fs -m1 $homedev |logecho
df -h |logecho

#add-apt-repository -y ppa:zfs-native/stable
add-apt-repository -y ppa:danielrichter2007/grub-customizer


# These do not require input
apt-get update 

logecho 'o Installing necessary packages...' 
apt-get -y install joe mc screen build-essential zfsutils-linux samba cifs-utils  \
    ethtool gparted lzop bwm-ng sysstat apt-file growisofs pv pbzip2 smartmontools p7zip-full fsarchiver \
    openssh-server autossh ntpdate ethstatus mingetty \
    lxde rxvt xterm \
    grub-customizer \
  |logecho

date
logecho 'o Syncing with timeserver...'
/usr/sbin/ntpdate -s pool.ntp.org
date


# housekeeping
mkdir -pv /root/tmpdel /root/dnld/Installed #; chmod 700 /root
mkdir -pv /home/dave/tmpdel;chown dave /home/dave/tmpdel

fdisk -l > /root/fdisk-l.txt
blkid |sort >> /root/fdisk-l.txt
# ls disk by id, no partitions, only certain fields, get rid of '../'
/bin/ls -l /dev/disk/by-id/ \
  | /bin/grep -v part \
  | /usr/bin/awk '{ print $9" "$10" "$11 }' \
  | /bin/sed 's%../%%g' \
  >> /root/fdisk-l.txt

# log smartctl info for all drives
mf=/root/smartctl.txt
> $mf 
for d in /dev/sd?; do
  echo "-- Processing $d" >> $mf
  blockdev --setra 16384 $d 
  blockdev --getra $d >> $mf
  /usr/sbin/smartctl --smart=on $d
  /usr/sbin/smartctl -a $d |head -n 16 \
    >> $mf
done


#stop tty3
#stop tty4
#stop tty5
#stop tty6
#start tty9
#mkdir -pv /root/disabled/etc-init
#( cd /etc/init; mv -v tty{4,5,6}.conf /root/disabled/etc-init )
# TODO nice tty9

logecho 'Updating existing packages' #;read -n 1
apt-get -y upgrade |logecho
apt-get -y autoremove |logecho

apt-file update 2>&1 > /dev/null &


logecho '** SYNCING **'
time sync;

free
echo 1 > /proc/sys/vm/drop_caches # free pagecache
free

logecho '! Done with initial setup - TODO: update root/bin/getdrive-byids and etc/rc.local;'
logecho 'TODO set static IP, update etc/hosts'
logecho '- TODO upgrade kernel, BACKUP, grub-customizer'

modprobe zfs
zpool status
zpool import |logecho

exit;

GRUB_CMDLINE_LINUX_DEFAULT="nosplash consoleblank=0 max_loop=40 parport=0"


# zpool create -f -o ashift=12 -o autoexpand=on -O atime=off zredtera1 \
  mirror ata-WDC_WD10EFRX-68FYTN0_WD-WCC4J1NL656R ata-WDC_WD10EFRX-68FYTN0_WD-WCC4J6KTJC0J

# pool=zblue500comp; zfs create -o atime=off $pool/dv; chown dave /$pool/dv; ls -al /$pool/dv

# zfs set sharesmb=on zredtera1/dv
