#!/bin/bash


# =LLC= © (C)opyright 2016 Boojum Consulting LLC / Dave Bechtel, All rights reserved.
## NOTICE: Only Boojum Consulting LLC personnel may use or redistribute this code,
## Unless given explicit permission by the author - see http://www.boojumconsultingsa.com
#
# REQUIRES zdynpool1

# TESTING virtual pool failure/hotspare + resilver
zp=zdynpool1
FAILD=2;REPW=8
SPARED=7

chkalreadyfailed=`zpool status $zp|grep -c disk8`
if [ $chkalreadyfailed -gt 0 ];then 
 FAILD=8;REPW=2
fi
  
# failexit.mrg
function failexit () {
  echo '! Something failed! Code: '"$1 $2" # code # (and optional description)
  exit $1
}

(set -x
  zpool set autoreplace=on $zp) || failexit 99 "! Something failed with $zp - Run mkdynamic-grow-pool-boojum.sh to Create $zp"

cd /zdisks || failexit 101 "! Cannot cd /zdisks; does $zp exist?"
[ -e zdyndisk$FAILD ] || failexit 105 "! /zdisks/zdyndisk$FAILD does not exist! Run mkdynamic-grow-pool-boojum.sh to Create $zp before running $0 !"
[ -e zdyndisk$REPW ] || failexit 105 "! /zdisks/zdyndisk$REPW does not exist! Run mkdynamic-grow-pool-boojum.sh to Create $zp before running $0 !"
  
zpool status $zp  
ls -lh /zdisks/
printf "o PK to make $zp disk #$FAILD FAIL -- will end up Replacing with disk #$REPW -- ^C to quit!"; read -n 1   
(set -x
  time dd if=/dev/urandom of=/zdisks/zdyndisk$FAILD bs=1M count=50)

ls -lh /zdisks/
zpool status $zp
echo "o OK - we wrote random data onto disk #$FAILD - trainwreck in progress!"
printf 'o As you can see, nothing bad has been detected yet... but when we run a Scrub: PK';read -n 1

# scrubwatch $zp
zpool scrub $zp

zpool status $zp
#  pool: zdynpool1
# state: DEGRADED
#status: One or more devices could not be used because the label is missing or
#        invalid.  Sufficient replicas exist for the pool to continue
#        functioning in a degraded state.
#action: Replace the device using 'zpool replace'.
#   see: http://zfsonlinux.org/msg/ZFS-8000-4J
#  scan: scrub repaired 0 in 0h0m with 0 errors on Wed May 25 18:12:44 2016
#            /zdisks/zdyndisk3  ONLINE       0     0     0
#          mirror-1             DEGRADED     0     0     0
#            /zdisks/zdyndisk2  UNAVAIL      0     0     0  corrupted data
#        spares
#          /zdisks/zdyndisk7    AVAIL

echo '';printf "o PK to replace the failing disk (#$FAILD) with Temporary hotspare disk #$SPARED:"; read -n 1
(set -x
   time zpool replace $zp /zdisks/zdyndisk$FAILD /zdisks/zdyndisk$SPARED)

zpool status $zp
#        NAME                     STATE     READ WRITE CKSUM
#        zdynpool1                DEGRADED     0     0     0
#          mirror-1               DEGRADED     0     0     0
#            spare-0              UNAVAIL      0     0     0
#              /zdisks/zdyndisk2  UNAVAIL      0     0     0  corrupted data
#              /zdisks/zdyndisk7  ONLINE       0     0     0
#            /zdisks/zdyndisk4    ONLINE       0     0     0
#        spares
#          /zdisks/zdyndisk7      INUSE     currently in use

echo '';echo 'o Now we can see the pool has resilvered the failing disk data (ONLY the in-use data, not whole disk) onto the hotspare;'
echo 'o NOTE the pool is still in a DEGRADED state until/unless we Replace the failing disk with a new disk...'
echo '+ OR we could issue a Detach for the failing disk! '
echo "o If we merely detach Disk#$FAILD at this point, Spare Disk#$SPARED will become the Permanent Replacement and we will NOT"
echo "+ have a hotspare available for ANY pool where $SPARED was allocated as a Spare - Hotspares can be shared across pools!"
printf "PK to replace Temp Hotspare disk #$SPARED with Perm Replacement disk #$REPW (This will also auto-send #$SPARED back to Hotspare state:)";read -n 1

(set -x
  time zpool replace $zp /zdisks/zdyndisk$FAILD /zdisks/zdyndisk$REPW)
  
zpool status $zp
printf `date +%H:%M:%S`' ...waiting for resilver to complete...'
waitresilver=1
while [ $waitresilver -gt 0 ];do 
 waitresilver=`zpool status $zp |grep -c INUSE`
 sleep 2
done 
date

echo 'o OK - we have FULLY RECOVERED from the trainwreck - with NO data loss!' 
printf 'PK to see the pool after recovery:';read -n 1
zpool status $zp
zfs list $zp
zpool list $zp
df -h |grep z

echo '';echo "o Note that the failing disk #$FAILD has now been permanently replaced with #$REPW and the Hotspare disk #$SPARED is again available."
echo " $zp is now back to normal, with NO data loss and NO downtime, and the pool is no longer in DEGRADED state."
echo 'Returning vdisks to normal size...'

zpool remove $zp /zdisks/zdyndisk$SPARED && zpool labelclear /zdisks/zdyndisk$SPARED
cp -v /zdisks/zdyndisk$SPARED /zdisks/zdyndisk$FAILD
zpool add $zp spare /zdisks/zdyndisk$SPARED 

ls -lh /zdisks/
zpool status $zp
echo 'o Complete!'

exit;

