#!/bin/bash

# TODO this is $oldcode, update with stuff from mkdynamic
# zfs pools on loop files

# =LLC= © (C)opyright 2016 Boojum Consulting LLC / Dave Bechtel, All rights reserved.
## NOTICE: Only Boojum Consulting LLC personnel may use or redistribute this code,
## Unless given explicit permission by the author - see http://www.boojumconsultingsa.com
#

mkdir -pv /zdisks
cd /zdisks || exit 99

newdisks=0 # set 1 to zero-out new disk files

# NOTE we need more loop devices; 1-time fix REF: http://askubuntu.com/questions/499131/how-to-use-more-than-255-loop-devices

logfile=~/mktestpool.log
> $logfile # clearit

function nomorepool () {
 zfs umount -f $1 2>> $logfile

# zpool export $1 2>> $logfile

 zpool status $1
 zpool destroy -f $1 2>> $logfile
 zpool status -L -P -x
}

pool1="zmirpool1"
pool2="zpoolraidz2"

pool3a="zsepdata1"
pool3b="zsepdata2"
pool3c="zsepdata3"

nomorepool $pool1
nomorepool $pool2

nomorepool $pool3a
nomorepool $pool3b
nomorepool $pool3c

zpool status
df
echo "ALL POOLS SHOULD BE GONE -PK"
[ $newdisks -gt 0 ] && read -n 1
[ $newdisks -gt 0 ] && /root/bin/clearcache


# getting rid of sync for each dd should speed things up
for i in {1..8};do
  printf $i...

# NECESSARY if re-using disks that were previously in a pool!!
  zpool labelclear -f /zdisks/zdmdisk$i
  zpool labelclear -f /zdisks/zdraid$i
  zpool labelclear -f /zdisks/zdmsepdata$i
  
[ $newdisks -gt 0 ] && time dd if=/dev/zero of=zdmdisk$i bs=1M count=400 2>&1 |egrep 'copied|real' >> $logfile
[ $newdisks -gt 0 ] && time dd if=/dev/zero of=zdraid$i bs=1M count=400 2>&1 |egrep 'copied|real' >> $logfile
[ $newdisks -gt 0 ] && time dd if=/dev/zero of=zdmsepdata$i bs=1M count=400 2>&1 |egrep 'copied|real' >> $logfile

done

time sync

ls -alh |tee -a $logfile

#NOTE: ' zfs add ' = add a disk (expand, non-redundant); ' zfs attach ' = add mirror

# REF: http://zfsonlinux.org/faq.html#WhatDevNamesShouldIUseWhenCreatingMyPool
# REF: https://flux.org.uk/tech/2007/03/zfs_tutorial_1.html

zpool create -f -o ashift=12 -o autoexpand=on -O atime=off $pool1 \
  mirror /zdisks/zdmdisk1 /zdisks/zdmdisk2 \
  mirror /zdisks/zdmdisk3 /zdisks/zdmdisk4 \
  mirror /zdisks/zdmdisk5 /zdisks/zdmdisk6 \
  spare /zdisks/zdmdisk7 /zdisks/zdmdisk8

# REF: https://docs.oracle.com/cd/E19120-01/open.solaris/817-2271/gcvcw/index.html
# Create raidz2 (still continues/fully acccessible despite 2-drive failure) 
# + and 1 hotspare

zpool create -f -o ashift=12 -o autoexpand=on -O atime=off $pool2 raidz2 \
  /zdisks/zdraid1 /zdisks/zdraid2 /zdisks/zdraid3 /zdisks/zdraid4 \
  /zdisks/zdraid5 /zdisks/zdraid6 \
  spare /zdisks/zdraid7 /zdisks/zdraid8


# 3 separate pools to separate data/sets, compression on for entire pool at creation time
zpool create -f -o ashift=12 -o autoexpand=on -O atime=off -O compression=lz4 $pool3a \
  mirror /zdisks/zdmsepdata1 /zdisks/zdmsepdata2
zpool create -f -o ashift=12 -o autoexpand=on -O atime=off -O compression=lz4 $pool3b \
  mirror /zdisks/zdmsepdata3 /zdisks/zdmsepdata4
zpool create -f -o ashift=12 -o autoexpand=on -O atime=off -O compression=lz4 $pool3c \
  mirror /zdisks/zdmsepdata5 /zdisks/zdmsepdata6
# 7 & 8 are unused/spares
 # zfs get all zsepdata1|grep compr
#zsepdata1  compressratio         1.00x                  -
#zsepdata1  compression           lz4                    local
#zsepdata1  refcompressratio      1.00x                  -


zpool status -L
#df -h |head -n 1 # unnec; fortuitously, grep also grabs "size"
df -h |grep z # /$pool1 /$pool2 /$pool3a /$pool3b /$pool3c

echo "Log: $logfile"

exit;

====================================================

# =LLC= © (C)opyright 2016 Boojum Consulting LLC / Dave Bechtel, All rights reserved.
## NOTICE: Only Boojum Consulting LLC personnel may use or redistribute this code,
## Unless given explicit permission by the author - see http://www.boojumconsultingsa.com
#
====================================================

Filesystem      Size  Used Avail Use% Mounted on
zmirpool1       1.1G     0  1.1G   0% /zmirpool1
zpoolraidz2     1.5G  256K  1.5G   1% /zpoolraidz2
zsepdata1       352M     0  352M   0% /zsepdata1
zsepdata2       352M     0  352M   0% /zsepdata2
zsepdata3       352M     0  352M   0% /zsepdata3

  pool: zmirpool1
 state: ONLINE
  scan: none requested
config:

        NAME                  STATE     READ WRITE CKSUM
        zmirpool1             ONLINE       0     0     0
          mirror-0            ONLINE       0     0     0
            /zdisks/zdmdisk1  ONLINE       0     0     0
            /zdisks/zdmdisk2  ONLINE       0     0     0
          mirror-1            ONLINE       0     0     0
            /zdisks/zdmdisk3  ONLINE       0     0     0
            /zdisks/zdmdisk4  ONLINE       0     0     0
          mirror-2            ONLINE       0     0     0
            /zdisks/zdmdisk5  ONLINE       0     0     0
            /zdisks/zdmdisk6  ONLINE       0     0     0
        spares
          /zdisks/zdmdisk7    AVAIL
          /zdisks/zdmdisk8    AVAIL

errors: No known data errors

  pool: zpoolraidz2
 state: ONLINE
  scan: none requested
config:

        NAME                 STATE     READ WRITE CKSUM
        zpoolraidz2          ONLINE       0     0     0
          raidz2-0           ONLINE       0     0     0
            /zdisks/zdraid1  ONLINE       0     0     0
            /zdisks/zdraid2  ONLINE       0     0     0
            /zdisks/zdraid3  ONLINE       0     0     0
            /zdisks/zdraid4  ONLINE       0     0     0
            /zdisks/zdraid5  ONLINE       0     0     0
            /zdisks/zdraid6  ONLINE       0     0     0
        spares
          /zdisks/zdraid7    AVAIL
          /zdisks/zdraid8    AVAIL

errors: No known data errors

  pool: zsepdata1
 state: ONLINE
  scan: none requested
config:

        NAME                     STATE     READ WRITE CKSUM
        zsepdata1                ONLINE       0     0     0
          mirror-0               ONLINE       0     0     0
            /zdisks/zdmsepdata1  ONLINE       0     0     0
            /zdisks/zdmsepdata2  ONLINE       0     0     0

errors: No known data errors

  pool: zsepdata2
 state: ONLINE
  scan: none requested
config:

        NAME                     STATE     READ WRITE CKSUM
        zsepdata2                ONLINE       0     0     0
          mirror-0               ONLINE       0     0     0
            /zdisks/zdmsepdata3  ONLINE       0     0     0
            /zdisks/zdmsepdata4  ONLINE       0     0     0

errors: No known data errors

  pool: zsepdata3
 state: ONLINE
  scan: none requested
config:

        NAME                     STATE     READ WRITE CKSUM
        zsepdata3                ONLINE       0     0     0
          mirror-0               ONLINE       0     0     0
            /zdisks/zdmsepdata5  ONLINE       0     0     0
            /zdisks/zdmsepdata6  ONLINE       0     0     0

errors: No known data errors



========================================================


# E no worky:
#zpool create -f -o ashift=12 -o autoexpand=on -O atime=off zpoolraidz2 raidz2 \
#  /zdisks/zraid1 /zdisks/zraid2 /zdisks/zraid3 /zdisks/zraid4 /zdisks/zraid5 /zdisks/zraid6 \
#  cache /zdisks/zraid7 \
#  spare /zdisks/zraid8
# error: cannot create 'zpoolraidz2': cache device must be a disk or disk slice

