#!/bin/bash
##Script to create encrypted ZFS dataset
##Tested on Ubuntu 18.04LTS with ZFS on Linux 0.8.0
## https://blog.heckel.xyz/2017/01/08/zfs-encryption-openzfs-zfs-on-linux/

#set -e
#set -x

source ~/bin/failexit.mrg

##Check for root priviliges
if [ "$(id -u)" -ne 0 ]; then
   echo "Please run as root."
   exit 1
fi

##define variables
# EDITME
#firstrun=0
#pointrel="0.8.0-rc3"

user=dave

#poolmount="/$poolname"
poolname=zmac5miltera2
dset=notshrencr

#DISKID=/mnt/imacdual/zdisk1
#DISKID=/zmac5miltera2

# Key location
mount /mnt/imacdual
[ `df |grep -c imacdual` -gt 0 ] || failexit 99 "! /mnt/imacdual not mounted!"

#zfskeyloc=/home/"$user"/zfskey
zfskeyloc=/mnt/imacdual/zek-$poolname

zpool set feature@encryption=enabled $poolname


createdatapool(){
	
	#generate 32 byte passkey (not 64) - cannot create 'zmac5miltera2/notshrencr': Raw key too long (expected 32).
	# NOTE rm keyfile if re-creating!
	[ -e $zfskeyloc ] || dd if=/dev/urandom of="$zfskeyloc" bs=32 count=1
	
	##for description of options see section 2.4b:
	##https://github.com/zfsonlinux/zfs/wiki/Debian-Buster-Encrypted-Root-on-ZFS
	##use create -n for dry-run
#	zpool create -O mountpoint="$poolmount"\
	zfs create \
		-o encryption=aes-128-gcm \
		-o keyformat=raw \
		-o keylocation=file://"$zfskeyloc" \
		-o compression=lz4 \
		-o sharesmb=off \
		-o acltype=posixacl \
		-o normalization=formD \
		-o atime=off \
		-o xattr=sa \
		$poolname/$dset
#		$poolname /dev/disk/by-id/"$DISKID"
		
	##set mountpoint permissions
	[ -e /$poolname/$dset ] && chown -R $user:$user "/$poolname/$dset"
	
	##bug in 0.8.0. zfs-mount.service mount command should have -l to load keyfile and automount pool at boot.
	##ExecStart line in /usr/lib/systemd/system/zfs-mount.service should be: "ExecStart=/sbin/zfs mount -l -a" 
	##https://github.com/zfsonlinux/zfs/issues/8750
	
	zpool status -v |awk 'NF>0'
	zfs mount -a
	zfs list
	df -T -h |head -n 1
	df -T -h |grep zfs
	
}

##call functions
##--------		

createdatapool


exit;

When a pool is imported using zpool import, encrypted datasets are left
unmounted, because their keys are not automatically loaded.  Only if the -l
option is passed will encrypted datasets be loaded (if they can):

Importing a pool with -l to load the keys:

# zpool import testpool -d . -l
Enter passphrase for 'testpool/enc1': (enter password)
Key load error: Failed to open key material file  # << Key file does not exist!
Enter passphrase for 'testpool/enc4': (enter password)
# zpool import testpool -d . -l

Instead of using zpool import -l ..., you can manually load the keys for individual datasets and volumes using zfs key -l:

Manually loading and unloading a key using 'zfs key -l'
# zfs key -l testpool/enc4
Enter passphrase for 'testpool/enc4': (enter password)

# zfs mount testpool/enc4
# Does not prompt. Just mounts!
# zfs key -l testpool/enc4
Enter passphrase for 'testpool/enc4': (enter password)
 
# zfs mount testpool/enc4
# Does not prompt. Just mounts!

Unloading a key of a mounted dataset wont work, because its still in use. The dataset has to be unmounted first:

# Works like a charm
# zfs umount testpool/enc4
# zfs key -u testpool/enc4

