#!/bin/bash
##Adapted Script to make zfs encrypted dataset
##Tested on Ubuntu 18.04LTS with ZFS on Linux 0.8.1
##ZFS on Linux release page: https://github.com/zfsonlinux/zfs/releases

#set -e
set -x

##define variables
# EDITME
user=dave

# should already exist and be zpool-upgraded
zp=zsam52

zfskeyloc=/mnt/imacdual/zek-$zp


##Check for root priviliges
if [ "$(id -u)" -ne 0 ]; then
   echo "Please run as root."
   exit 1
fi


##declare functions

mkencryptedataset(){
	#generate 32 byte passkey
	[ -e "$zfskeyloc" ] || dd if=/dev/urandom of="$zfskeyloc" bs=1 count=32
	
	##Get Disk UUID
#	ls -la /dev/disk/by-id
#	echo "Enter Disk ID (must match exactly):"
#	read DISKID
	#DISKID=<enter override here>
#	echo "Disk ID set to $DISKID"

	##create pool
	##for description of options see section 2.4b:
	##https://github.com/zfsonlinux/zfs/wiki/Debian-Buster-Encrypted-Root-on-ZFS
	##Note options with -O are file-system-properties. options with -o aren't. need to use upper and lowercase correctly.
	##use create -n for dry-run
#	-O mountpoint="$poolmount"\
#	zpool create \
	zfs create \
		-o encryption=aes-128-gcm \
		-o keyformat=raw \
		-o keylocation=file://"$zfskeyloc" \
		-o compression=lz4 \
		-o acltype=posixacl \
		-o normalization=formD \
		-o relatime=off \
		-o atime=off \
		-o xattr=sa \
		-o recordsize=512k \
		-o sharesmb=on \
		$zp/shrencr
		
	##bug in 0.8.0. zfs-mount.service mount command should have -l to load keyfile and automount pool at boot.
	##ExecStart line in /usr/lib/systemd/system/zfs-mount.service should be: "ExecStart=/sbin/zfs mount -l -a" 
	##https://github.com/zfsonlinux/zfs/issues/8750

	zfs load-key -a
	zfs mount -a

	##set mountpoint permissions
	[ -e "$zp/shrencr" ] && chown -Rv $user:$user "/$zp/shrencr"
	
	echo 'PK to proceed'
	read -n 1
	
	(zpool status -v $zp
	zfs list |egrep "AVAIL|$zp"
	df -h -T ) |less
	
}

##call functions
##--------		
mkencryptedataset
