#!/bin/bash

# destroy 6x2TB mirror and recreate as raidz2, copy data over to new datasets

source ~/bin/failexit.mrg
shopt -s expand_aliases

alias pvv='pv -t -r -b -W -i 2 -B 200M '

zp=zseatera2
bp=zwd6t 
 # backup pool
 
zpool create -f -o ashift=12 -o autoexpand=on -O atime=off -O compression=lz4 \
   $zp raidz2 \
ata-ST2000VN000-1HJ164_W72127JB \
ata-ST2000VN000-1HJ164_W7212LTE \
ata-ST2000VN000-1HJ164_W523GA5J \
ata-ST2000VN000-1HJ164_W5238TSL \
ata-ST2000VN000-1HJ164_Z520DLXJ \
ata-ST2000VN004-2E4164_Z521TRH4

[ `df |grep -c $zp` -gt 0 ] || failexit 101 "$zp not found after create"

zfs-newds.sh 11 $zp shrcompr
zfs-newds.sh 10 $zp notshrcompr
zfs-newds.sh 10 $zp bkp-vmware-vms--p3300lts
zfs-newds.sh 10 $zp bkp-virtualbox-vms--p2700quad1404
zfs-newds.sh 10 $zp from-p3300-vbox-bkp-20180426-virtbox-virtmachines
zfs-newds.sh 10 $zp from-p3300-zbkpteratmp-compr
zfs-newds.sh 10 $zp vboxtest

#b4 raidz2:                                                                                                                         # dfh /zseatera2/*                                                                                                                 Filesystem                                                  Type  Size  Used Avail Use% Mounted on                                 zseatera2                                                   zfs   2.2T  128K  2.2T   1% /zseatera2
#zseatera2/dv                                                2.4T  162G  2.2T   7% /zseatera2/dv
# dv/shrcompr
#zseatera2/from-p3300-vbox-bkp-20180426-virtbox-virtmachines 2.7T  497G  2.2T  19% /zseatera2/from-p3300-vbox-bkp-20180426-vir
#zseatera2/notshrcompr                                       3.0T  841G  2.2T  28% /zseatera2/notshrcompr
#zseatera2/vboxtest                                          2.2T  2.5G  2.2T   1% /zseatera2/vboxtest

#zwd6t                                                                  2.1T  128K  2.1T   1% /zwd6t
#zwd6t/dv-shrcompr                                                      2.8T  713G  2.1T  26% /zwd6t/dv-shrcompr
#zwd6t/from-p3300-zbkpteratmp-compr                                     2.2T   99G  2.1T   5% /zwd6t/from-p3300-zbkpteratmp-compr
#zwd6t/dv-shrcompr/bkp-vmware-vms--p3300lts                             2.8T  694G  2.1T  25% /zwd6t/dv-shrcompr/bkp-vmware-vms--p3300lts

#zwd6t/from-zseatera2                                                   2.1T  1.0M  2.1T   1% /zwd6t/from-zseatera2
#zwd6t/from-zseatera2/dv                                                2.3T  157G  2.1T   7% /zwd6t/from-zseatera2/dv
#zwd6t/from-zseatera2/from-p3300-vbox-bkp-20180426-virtbox-virtmachines 2.6T  497G  2.1T  19% /zwd6t/from-zseatera2/from-p3300-vbox-bkp-20180426-virtbox-virtmachines
#zwd6t/from-zseatera2/notshrcompr                                       2.9T  840G  2.1T  29% /zwd6t/from-zseatera2/notshrcompr
#zwd6t/from-zseatera2/notshrcompr/bkp-virtualbox-vms--p2700quad1404     2.4T  285G  2.1T  12% /zwd6t/from-zseatera2/notshrcompr/bkp-virtualbox-vms--p2700quad1404
#zwd6t/from-zseatera2/vboxtest                                          2.1T  2.5G  2.1T   1% /zwd6t/from-zseatera2/vboxtest

# manual copy
errfile=~/$zp-copy-errs.log
> $errfile

date > $errfile
alias dfh='df -T -h' >> $errfile


# args: $1 = src , $2 = dest
function mancp () {
  echo "`date` - Starting $1 _to_ $2" >> $errfile
  cd $1 && time tar cpf - * |pvv |(cd $2 && tar xpf -) 2>>$errfile
  time sync
  sleep 5
}


mancp /$bp/from-zseatera2/vboxtest  /$zp/vboxtest
mancp /$bp/from-p3300-zbkpteratmp-compr  /$zp/from-p3300-zbkpteratmp-compr
mancp /$bp/from-zseatera2/dv  /$zp/shrcompr
mancp /$bp/from-zseatera2bkp-virtualbox-vms--p2700quad1404  /$zp/bkp-virtualbox-vms--p2700quad1404
mancp /$bp/from-zseatera2/from-p3300-vbox-bkp-20180426-virtbox-virtmachines /$zp/from-p3300-vbox-bkp-20180426-virtbox-virtmachines

mancp /$bp/bkp-vmware-vms--p3300lts  /$zp/bkp-vmware-vms--p3300lts

mancp /$bp/dv-shrcompr  /$zp/shrcompr
mancp /$bp/from-zseatera2/notshrcompr  /$zp/notshrcompr 

# fallback: rsync
date >> $errfile


exit;

# do smallest 1st
# dfh |grep zfs |sort -h -k 4
zseatera2/vboxtest                                                     zfs       3.6T  2.5G  3.6T   1% /zseatera2/vboxtest
zwd6t/from-zseatera2/vboxtest                                          zfs       2.1T  2.5G  2.1T   1% /zwd6t/from-zseatera2/vboxtest

zwd6t/from-p3300-zbkpteratmp-compr                                     zfs       2.2T   99G  2.1T   5% /zwd6t/from-p3300-zbkpteratmp-compr

zwd6t/from-zseatera2/dv                                                zfs       2.3T  157G  2.1T   7% /zwd6t/from-zseatera2/dv
zseatera2/dv                                                           zfs       3.7T  162G  3.6T   5% /zseatera2/dv
# newdest: shrcompr

# DONE zfs rename (rid notshrompr) so we dont descend and dup!!
zseatera2/notshrcompr/bkp-virtualbox-vms--p2700quad1404                zfs       3.9T  285G  3.6T   8% /zseatera2/notshrcompr/bkp-virtualbox-vms--p2700quad1404
zwd6t/from-zseatera2/notshrcompr/bkp-virtualbox-vms--p2700quad1404     zfs       2.4T  285G  2.1T  12% /zwd6t/from-zseatera2/notshrcompr/bkp-virtualbox-vms--p2700quad1404
# newdest: moving up to root
# zfs rename zwd6t/from-zseatera2/notshrcompr/bkp-virtualbox-vms--p2700quad1404 zwd6t/from-zseatera2bkp-virtualbox-vms--p2700quad1404
zwd6t/from-zseatera2bkp-virtualbox-vms--p2700quad1404                  zfs       2.4T  285G  2.1T  12% /zwd6t/from-zseatera2bkp-virtualbox-vms--p2700quad1404


zseatera2/from-p3300-vbox-bkp-20180426-virtbox-virtmachines            zfs       4.1T  497G  3.6T  13% /zseatera2/from-p3300-vbox-bkp-20180426-virtbox-virtmachines
zwd6t/from-zseatera2/from-p3300-vbox-bkp-20180426-virtbox-virtmachines zfs       2.6T  497G  2.1T  19% /zwd6t/from-zseatera2/from-p3300-vbox-bkp-20180426-virtbox-virtmachines

# DONE zfs rename (rid dv-shrcompr) so we dont descend and dup!!
zwd6t/dv-shrcompr/bkp-vmware-vms--p3300lts                             zfs       2.8T  694G  2.1T  25% /zwd6t/dv-shrcompr/bkp-vmware-vms--p3300lts
# newdest: moving up to root
# zfs rename zwd6t/dv-shrcompr/bkp-vmware-vms--p3300lts  zwd6t/bkp-vmware-vms--p3300lts 
zwd6t/bkp-vmware-vms--p3300lts                                         zfs       2.8T  694G  2.1T  25% /zwd6t/bkp-vmware-vms--p3300lts


zwd6t/dv-shrcompr                                                      zfs       2.8T  713G  2.1T  26% /zwd6t/dv-shrcompr
# newdest: moving to shrcompr

zwd6t/from-zseatera2/notshrcompr                                       zfs       2.9T  840G  2.1T  29% /zwd6t/from-zseatera2/notshrcompr
zseatera2/notshrcompr                                                  zfs       4.4T  841G  3.6T  19% /zseatera2/notshrcompr
